-----------------------------------------------------------------------------------------------
-- Client Lua Script for Greedy
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
require "Item"
 
-----------------------------------------------------------------------------------------------
-- Greedy Module Definition
-----------------------------------------------------------------------------------------------
local Greedy = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local eGreedy = {}
eGreedy.greed = 1
eGreedy.need = 2
eGreedy.disabled = 3

local tGreedySettings = { 
	"nGreedEquipment",
	"nGreedMisc",
	"nGreedRunes",
	"btRuneTypes",
	"bEquipPass",
	"eGreedQuality"
}

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Greedy:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
	
	o.restored = false
	o.needFlag = false
	
    return o
end

function Greedy:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "Greedy"
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- Greedy OnLoad
-----------------------------------------------------------------------------------------------
function Greedy:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("Greedy.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- Greedy OnDocLoaded
-----------------------------------------------------------------------------------------------
function Greedy:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "GreedyForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("greedy", 				"OnGreedyOn", self)
		Apollo.RegisterEventHandler("LootRollUpdate",		"OnGroupLoot", self)
		
		--look up imporant form elements now (so we don't have to search again each time)
		self.tUI = {}
		self.tUI.wndEquipment = self.wndMain:FindChild("EquipmentFrame")
		self.tUI.wndRunes = self.wndMain:FindChild("RuneFrame")
		self.tUI.wndMisc = self.wndMain:FindChild("MiscFrame")
		self.tUI.wndRarity = self.wndMain:FindChild("RarityTypesFrame")
		
		self.tUI.btnAir = self.wndMain:FindChild("RuneAirButton")
		self.tUI.btnEarth = self.wndMain:FindChild("RuneEarthButton")
		self.tUI.btnFire = self.wndMain:FindChild("RuneFireButton")
		self.tUI.btnFusion = self.wndMain:FindChild("RuneFusionButton")
		self.tUI.btnLife = self.wndMain:FindChild("RuneLifeButton")
		self.tUI.btnLogic = self.wndMain:FindChild("RuneLogicButton")
		self.tUI.btnWater = self.wndMain:FindChild("RuneWaterButton")
		
		self.tUI.btnEquipPass = self.wndMain:FindChild("EquipmentPassButton")
		
		--set up converter tables
		self.tUI.BtnToQuality = {}
		self.tUI.BtnToQuality[1] = Item.CodeEnumItemQuality.Average
		self.tUI.BtnToQuality[2] = Item.CodeEnumItemQuality.Good
		self.tUI.BtnToQuality[3] = Item.CodeEnumItemQuality.Excellent
		self.tUI.BtnToQuality[4] = Item.CodeEnumItemQuality.Superb
		self.tUI.BtnToQuality[5] = Item.CodeEnumItemQuality.Legendary
		self.tUI.BtnToQuality[6] = Item.CodeEnumItemQuality.Artifact
		
		self.tUI.QualityToBtn = {}
		self.tUI.QualityToBtn[Item.CodeEnumItemQuality.Average] = 1
		self.tUI.QualityToBtn[Item.CodeEnumItemQuality.Good] = 2
		self.tUI.QualityToBtn[Item.CodeEnumItemQuality.Excellent] = 3
		self.tUI.QualityToBtn[Item.CodeEnumItemQuality.Superb] = 4
		self.tUI.QualityToBtn[Item.CodeEnumItemQuality.Legendary] = 5
		self.tUI.QualityToBtn[Item.CodeEnumItemQuality.Artifact] = 6
		
		self.tUI.ItemTypeToSigilEnum = {}
		self.tUI.ItemTypeToSigilEnum[339] = Item.CodeEnumSigilType.Water
		self.tUI.ItemTypeToSigilEnum[340] = Item.CodeEnumSigilType.Life
		self.tUI.ItemTypeToSigilEnum[341] = Item.CodeEnumSigilType.Earth
		self.tUI.ItemTypeToSigilEnum[342] = Item.CodeEnumSigilType.Fusion
		self.tUI.ItemTypeToSigilEnum[343] = Item.CodeEnumSigilType.Fire
		self.tUI.ItemTypeToSigilEnum[344] = Item.CodeEnumSigilType.Logic
		self.tUI.ItemTypeToSigilEnum[345] = Item.CodeEnumSigilType.Air
		
		
		-- Do additional Addon initialization here
		self.crbAddon = Apollo.GetAddon("NeedVsGreed")
		self.crbChat = Apollo.GetAddon("ChatLog")
		
		if GameLib.GetLootRolls() then
			self:OnGroupLoot()
		end
	end
end

-----------------------------------------------------------------------------------------------
-- Greedy Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function Greedy:OnGroupLoot()
	--look for settings to be sure
	if not self.restored then
		self:DefaultSettings()
	end
	
	--get list of loot
	local tLoot = GameLib.GetLootRolls()
	
	--greed on everything that fits the criteria
	for idx, tCurrentElement in ipairs(tLoot) do
		self.needFlag = false
		
		if self:IsGreedyOn(tCurrentElement) then
			local nLootID = tCurrentElement.nLootId
			if self.needFlag == 3 then
				GameLib.PassOnLoot(nLootID)
			else
				GameLib.RollOnLoot(nLootID, self.needFlag)
			end
			

			
			--possibly inform crbAddon
			local crbWindow = self.crbAddon.wndMain
			if not crbWindow then return end
			if crbWindow:GetData() == nLootID then
				crbWindow:Close()
			end
		end
		
	end
end

-- check if we want the item and store in bNeed if we want to need it.
function Greedy:IsGreedyOn(tLoot)
	local tItem = tLoot.itemDrop
		
	--equipment
	if tItem:IsEquippable() then
		if self.bEquipPass then self.needFlag = 3 end	--set pass toggle for gear
		
		if self.nGreedEquipment == eGreedy.need then
			return (not GameLib.IsNeedRollAllowed(tLoot.nLootId))
		elseif self.nGreedEquipment == eGreedy.greed then
			return not (self.eGreedQuality > tItem:GetItemQuality())
		end
	end
	
	--runes
	if self.nGreedRunes < eGreedy.disabled then
		if tItem:GetItemType() <= 345 and tItem:GetItemType() >= 339  then
			if self.nGreedRunes == eGreedy.need then
				self.needFlag = true
			end
			
			return self.btRuneTypes[self.tUI.ItemTypeToSigilEnum[tItem:GetItemType()]]
		end
	end
	
	--misc
	if self.nGreedMisc < eGreedy.disabled then
		if self.nGreedMisc == eGreedy.need then self.needFlag = true end
		
		return true
	end
	
	return false --if nothing else latches on, we dont wanna roll
end

-- for the button in options menu
function Greedy:OnConfigure()
	self:OnGreedyOn()
end

-- set default settings
function Greedy:DefaultSettings()
	self.nGreedEquipment = eGreedy.need
	self.nGreedMisc = eGreedy.disabled
	self.nGreedRunes = eGreedy.greed

	self.btRuneTypes = {}
	self.btRuneTypes[Item.CodeEnumSigilType.Air] = false
	self.btRuneTypes[Item.CodeEnumSigilType.Earth] = false
	self.btRuneTypes[Item.CodeEnumSigilType.Fire] = false
	self.btRuneTypes[Item.CodeEnumSigilType.Water] = false
	self.btRuneTypes[Item.CodeEnumSigilType.Fusion] = false
	self.btRuneTypes[Item.CodeEnumSigilType.Life] = false
	self.btRuneTypes[Item.CodeEnumSigilType.Logic] = false
	
	self.bEquipPass = false
	
	self.eGreedQuality = Item.CodeEnumItemQuality.Good
	
	self.restored = true
end

-- Save User Settings
function Greedy:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end

	local tSave = {}
	for idx,property in ipairs(tGreedySettings) do tSave[property] = self[property] end
	
	return tSave
end


-- Restore Saved User Settings
function Greedy:OnRestore(eType, t)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	
	for idx,property in ipairs(tGreedySettings) do
		if t[property] ~= nil then self[property] = t[property] end
	end
	
	self.restored = true
end



-----------------------------------------------------------------------------------------------
-- GreedyForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function Greedy:OnApply()
	--save the settings.
	self.nGreedEquipment = self.tUI.wndEquipment:GetRadioSel("Equipment_ButtonGroup")
	self.nGreedMisc = self.tUI.wndMisc:GetRadioSel("Misc_ButtonGroup")
	self.nGreedRunes = self.tUI.wndRunes:GetRadioSel("Rune_ButtonGroup")
	
	self.btRuneTypes[Item.CodeEnumSigilType.Air] = self.tUI.btnAir:IsChecked()
	self.btRuneTypes[Item.CodeEnumSigilType.Earth] = self.tUI.btnEarth:IsChecked()
	self.btRuneTypes[Item.CodeEnumSigilType.Fire] = self.tUI.btnFire:IsChecked()
	self.btRuneTypes[Item.CodeEnumSigilType.Fusion] = self.tUI.btnFusion:IsChecked()
	self.btRuneTypes[Item.CodeEnumSigilType.Life] = self.tUI.btnLife:IsChecked()
	self.btRuneTypes[Item.CodeEnumSigilType.Logic] = self.tUI.btnLogic:IsChecked()
	self.btRuneTypes[Item.CodeEnumSigilType.Water] = self.tUI.btnWater:IsChecked()
	
	self.eGreedQuality = self.tUI.BtnToQuality[self.tUI.wndRarity:GetRadioSel("Rarity_RadioGroup")]
	self.bEquipPass = self.tUI.btnEquipPass:IsChecked()
	
	self.wndMain:Close() -- hide the window
end

function Greedy:OnAllRunes()
	if self.tUI.btnAir:IsChecked() then --dont use air as dirty toggle check
		self.tUI.btnAir:SetCheck(false)
		self.tUI.btnEarth:SetCheck(false)
		self.tUI.btnFire:SetCheck(false)
		self.tUI.btnFusion:SetCheck(false)
		self.tUI.btnLife:SetCheck(false)
		self.tUI.btnLogic:SetCheck(false)
		self.tUI.btnWater:SetCheck(false)
	else
		self.tUI.btnAir:SetCheck(true)
		self.tUI.btnEarth:SetCheck(true)
		self.tUI.btnFire:SetCheck(true)
		self.tUI.btnFusion:SetCheck(true)
		self.tUI.btnLife:SetCheck(true)
		self.tUI.btnLogic:SetCheck(true)
		self.tUI.btnWater:SetCheck(true)
	end
end

-- when the Cancel button is clicked
function Greedy:OnCancel()
	self.wndMain:Close() -- hide the window
end

-- on SlashCommand "/greedy" or opening via options menu
function Greedy:OnGreedyOn()
	--look for settings to be sure
	if not self.restored then
		self:DefaultSettings()
	end
	--make the UI show current settings.
	--Equip settings
	self.tUI.wndEquipment:SetRadioSel("Equipment_ButtonGroup",self.nGreedEquipment)
	self.tUI.wndRarity:SetRadioSel("Rarity_RadioGroup", self.tUI.QualityToBtn[self.eGreedQuality])
	
	--Misc settings
	self.tUI.wndMisc:SetRadioSel("Misc_ButtonGroup", self.nGreedMisc)
	
	--rune settings
	self.tUI.wndRunes:SetRadioSel("Rune_ButtonGroup", self.nGreedRunes)
	
	self.tUI.btnAir:SetCheck(self.btRuneTypes[Item.CodeEnumSigilType.Air])
	self.tUI.btnEarth:SetCheck(self.btRuneTypes[Item.CodeEnumSigilType.Earth])
	self.tUI.btnFire:SetCheck(self.btRuneTypes[Item.CodeEnumSigilType.Fire])
	self.tUI.btnFusion:SetCheck(self.btRuneTypes[Item.CodeEnumSigilType.Fusion])
	self.tUI.btnLife:SetCheck(self.btRuneTypes[Item.CodeEnumSigilType.Life])
	self.tUI.btnLogic:SetCheck(self.btRuneTypes[Item.CodeEnumSigilType.Logic])
	self.tUI.btnWater:SetCheck(self.btRuneTypes[Item.CodeEnumSigilType.Water])
	
	self.tUI.btnEquipPass:SetCheck(self.bEquipPass)
	
	self.wndMain:Invoke() -- show the window
end


-----------------------------------------------------------------------------------------------
-- Greedy Instance
-----------------------------------------------------------------------------------------------
local GreedyInst = Greedy:new()
GreedyInst:Init()
